/* eslint-disable react/display-name */
import { Accordion } from '@material-ui/core'
import Button from '@material-ui/core/Button'
import AnalysisConsumption from 'components/Analysis/AnalysisConsumption'
import { FluidType } from 'enum/fluid.enum'
import { mount } from 'enzyme'
import { PerformanceIndicator } from 'models'
import React from 'react'
import { Provider } from 'react-redux'
import configureStore from 'redux-mock-store'
import { globalStateData } from '../../../tests/__mocks__/globalStateData.mock'
import { profileData } from '../../../tests/__mocks__/profileData.mock'
import {
  mockMonthlyForecastJanuaryTestProfile1,
  profileTypeData,
} from '../../../tests/__mocks__/profileType.mock'
import { mockAnalysisState } from '../../../tests/__mocks__/store'
import { waitForComponentToPaint } from '../../../tests/__mocks__/testUtils'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

const mockgetMonthlyForecast = jest.fn()
jest.mock('services/profileType.service', () => {
  return jest.fn(() => {
    return {
      getMonthlyForecast: mockgetMonthlyForecast,
    }
  })
})

const mockgetProfileTypeData = jest.fn()
jest.mock('services/profileTypeEntity.service', () => {
  return jest.fn(() => {
    return {
      getProfileType: mockgetProfileTypeData,
    }
  })
})

jest.mock('components/Analysis/AnalysisConsumptionRow', () => () => (
  <div id="analysisconsumptionrow" />
))

const modifiedProfile = { ...profileData, isProfileTypeCompleted: true }
const mockStore = configureStore([])
const store = mockStore({
  ecolyo: {
    profile: modifiedProfile,
    profileType: profileTypeData,
    global: globalStateData,
    analysis: mockAnalysisState,
  },
})

const performanceIndicator = {
  compareValue: 160.42797399999998,
  percentageVariation: 0.026592126632478563,
  value: 156.161853,
}
const performanceIndicators = [
  {
    compareValue: 203.49,
    percentageVariation: 0.12261044768784712,
    value: 178.54,
  },
  {
    compareValue: 7926.82,
    percentageVariation: 0.020542916327102145,
    value: 7763.98,
  },
  {
    compareValue: 1316.46,
    percentageVariation: -0.0009191316105312541,
    value: 1317.67,
  },
]

describe('AnalysisConsumption component', () => {
  it('should be rendered correctly', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find(Accordion).exists()).toBeTruthy()
    expect(wrapper.find('#analysisconsumptionrow').length).toBe(4)
  })

  it('should be rendered correctly with no profil set', async () => {
    const mockAggregatedPerformanceIndicator: PerformanceIndicator = {
      compareValue: 160.42797399999998,
      percentageVariation: null,
      value: null,
    }
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={mockAggregatedPerformanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find(Button).exists()).toBeTruthy()
  })

  it('should be rendered correctly with null aggregated performance indicator', async () => {
    const mockAggregatedPerformanceIndicator: PerformanceIndicator = {
      compareValue: 160.42797399999998,
      percentageVariation: null,
      value: null,
    }
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={mockAggregatedPerformanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find(Accordion).exists()).toBeTruthy()
    expect(wrapper.find('#analysisconsumptionrow').length).toBe(4)
  })

  it('should be rendered correctly without fluid', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(
      wrapper.find('#analysisconsumptionrow').first().parent().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should be rendered correctly with all fluids connected', async () => {
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        profileType: profileTypeData,
        global: {
          fluidTypes: [FluidType.ELECTRICITY, FluidType.WATER, FluidType.GAS],
        },
        analysis: {
          ...mockAnalysisState,
          analysisMonth: profileData.monthlyAnalysisDate,
        },
      },
    })
    mockgetMonthlyForecast.mockReturnValue(
      mockMonthlyForecastJanuaryTestProfile1
    )
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(mockgetMonthlyForecast).toHaveBeenCalledWith(
      profileData.monthlyAnalysisDate.month - 1
    )
    expect(
      wrapper.find('#analysisconsumptionrow').first().parent().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should be rendered correctly with 2 fluids connected', async () => {
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        global: { fluidTypes: [FluidType.ELECTRICITY, FluidType.WATER] },
        analysis: { analysisMonth: profileData.monthlyAnalysisDate },
      },
    })
    mockgetMonthlyForecast.mockReturnValue(
      mockMonthlyForecastJanuaryTestProfile1
    )
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(mockgetMonthlyForecast).toHaveBeenCalledWith(
      profileData.monthlyAnalysisDate.month - 1
    )
    expect(
      wrapper.find('#analysisconsumptionrow').first().parent().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should redirect to profileType form when click on mui button', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    wrapper.find(Button).first().simulate('click')
    expect(mockedNavigate).toBeCalledWith('/profileType')
  })
})
