import { Button } from '@material-ui/core'
import Loader from 'components/Loader/Loader'
import FluidPerformanceIndicator from 'components/PerformanceIndicator/FluidPerformanceIndicator'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { TimeStep } from 'enum/timeStep.enum'
import { FluidConfig, PerformanceIndicator } from 'models'
import React, { Dispatch, useEffect, useMemo, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import ConsumptionService from 'services/consumption.service'
import { AppActionsTypes, AppStore } from 'store'
import { setPeriod } from 'store/analysis/analysis.slice'
import './comparisonView.scss'

const ComparisonView = ({ fluidConfig }: { fluidConfig: FluidConfig[] }) => {
  const { t } = useI18n()
  const client = useClient()
  const {
    global: { fluidTypes },
    analysis: { period, analysisMonth },
  } = useSelector((state: AppStore) => state.ecolyo)
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const [monthPerformanceIndicators, setMonthPerformanceIndicators] = useState<
    PerformanceIndicator[]
  >([])
  const [yearPerformanceIndicators, setYearPerformanceIndicators] = useState<
    PerformanceIndicator[]
  >([])
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const consumptionService = useMemo(
    () => new ConsumptionService(client),
    [client]
  )
  const monthPeriod = useMemo(() => {
    return {
      startDate: analysisMonth.minus({ month: 1 }).startOf('month'),
      endDate: analysisMonth.minus({ month: 1 }).endOf('month'),
    }
  }, [analysisMonth])
  const previousMonthPeriod = useMemo(() => {
    return {
      startDate: analysisMonth.minus({ month: 2 }).startOf('month'),
      endDate: analysisMonth.minus({ month: 2 }).endOf('month'),
    }
  }, [analysisMonth])
  const previousYearPeriod = useMemo(() => {
    return {
      startDate: analysisMonth.minus({ year: 1, month: 1 }).startOf('month'),
      endDate: analysisMonth.minus({ year: 1, month: 1 }).endOf('month'),
    }
  }, [analysisMonth])

  const loaderPlaceholderHeight =
    fluidTypes.length * 84 + (fluidTypes.length - 1) * 10

  useEffect(() => {
    let subscribed = true
    async function populateData() {
      if (subscribed) {
        const fetchedMonthIndicators =
          await consumptionService.getPerformanceIndicators(
            monthPeriod,
            TimeStep.MONTH,
            fluidTypes,
            previousMonthPeriod
          )
        const fetchedYearIndicators =
          await consumptionService.getPerformanceIndicators(
            monthPeriod,
            TimeStep.MONTH,
            fluidTypes,
            previousYearPeriod
          )
        if (fetchedMonthIndicators) {
          setMonthPerformanceIndicators(fetchedMonthIndicators)
        }
        if (fetchedYearIndicators) {
          setYearPerformanceIndicators(fetchedYearIndicators)
        }
        setIsLoading(false)
      }
    }
    populateData()

    return () => {
      subscribed = false
    }
  }, [
    client,
    fluidTypes,
    analysisMonth,
    consumptionService,
    monthPeriod,
    previousMonthPeriod,
    previousYearPeriod,
  ])

  return (
    <div className="comparisonView">
      <div className="tabs">
        <Button
          className={period === 'month' ? 'active' : ''}
          onClick={() => dispatch(setPeriod('month'))}
        >
          {t('analysis.compare.month_tab')}
        </Button>
        <Button
          className={period === 'year' ? 'active' : ''}
          onClick={() => dispatch(setPeriod('year'))}
        >
          {t('analysis.compare.year_tab')}
        </Button>
      </div>
      <div className="performanceIndicators">
        {isLoading && (
          <div
            style={{
              height: `${loaderPlaceholderHeight}px`,
              display: 'flex',
              justifyContent: 'center',
            }}
          >
            <Loader />
          </div>
        )}
        {!isLoading &&
          fluidConfig.map(
            fluid =>
              fluidTypes.includes(fluid.fluidTypeId) && (
                <FluidPerformanceIndicator
                  key={fluid.konnectorConfig.slug}
                  fluidType={fluid.fluidTypeId}
                  performanceIndicator={
                    period === 'month'
                      ? monthPerformanceIndicators[fluid.fluidTypeId]
                      : yearPerformanceIndicators[fluid.fluidTypeId]
                  }
                  comparisonDate={
                    period === 'month'
                      ? previousMonthPeriod.startDate
                      : previousYearPeriod.startDate
                  }
                />
              )
          )}
      </div>
    </div>
  )
}

export default ComparisonView
