import IconButton from '@material-ui/core/IconButton'
import GraphIcon from 'assets/icons/ico/graph-icon.svg'
import LeftArrowIcon from 'assets/icons/ico/left-arrow.svg'
import RightArrowIcon from 'assets/icons/ico/right-arrow.svg'
import BarChart from 'components/Charts/BarChart'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import DataloadSection from 'components/ConsumptionVisualizer/DataloadSection'
import { useChartResize } from 'components/Hooks/useChartResize'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { DataloadSectionType } from 'enum/dataload.enum'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { Datachart, Dataload, TimePeriod } from 'models'
import React, { Dispatch, useEffect, useRef, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import ConsumptionDataManager from 'services/consumption.service'
import { AppActionsTypes, AppStore } from 'store'
import { setSelectedDate } from 'store/chart/chart.slice'
import './maxConsumptionCard.scss'

const MaxConsumptionCard = () => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const {
    global: { fluidTypes },
    analysis: { analysisMonth },
  } = useSelector((state: AppStore) => state.ecolyo)
  const [index, setIndex] = useState<number>(0)
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [maxDayData, setMaxDayData] = useState<Dataload | null>(null)
  const [chartData, setChartData] = useState<Datachart>({
    actualData: [],
    comparisonData: null,
  })
  const containerRef = useRef<HTMLDivElement>(null)
  const { height, width } = useChartResize(containerRef, isLoading, 250, 940)

  const currentFluidType = FluidType[fluidTypes[index]] as
    | 'ELECTRICITY'
    | 'WATER'
    | 'GAZ'
  const fluidColor = currentFluidType.toLowerCase()

  const handleFluidChange = (direction: number) => {
    setIsLoading(true)
    let newIndex = index + direction
    if (newIndex >= fluidTypes.length) {
      newIndex = 0
    } else if (newIndex < 0) {
      newIndex = fluidTypes.length - 1
    }
    setIndex(newIndex)
  }

  useEffect(() => {
    let subscribed = true
    async function getMaxLoadData() {
      setIsLoading(true)
      const timePeriod: TimePeriod = {
        startDate: analysisMonth.minus({ month: 1 }).startOf('month'),
        endDate: analysisMonth.minus({ month: 1 }).endOf('month'),
      }
      const consumptionService = new ConsumptionDataManager(client)
      const monthlyData = await consumptionService.getGraphData(
        timePeriod,
        TimeStep.DAY,
        [fluidTypes[index]]
      )

      if (monthlyData && monthlyData?.actualData.length > 0) {
        setChartData(monthlyData)
        const maxDay = getMaxConsumptionDay(monthlyData.actualData)
        if (maxDay) {
          setMaxDayData(maxDay)
          dispatch(setSelectedDate(maxDay.date))
        } else {
          setMaxDayData(null)
        }
      }
      setIsLoading(false)
    }
    if (subscribed) {
      getMaxLoadData()
    }
    return () => {
      subscribed = false
    }
  }, [analysisMonth, client, fluidTypes, index, dispatch])

  const getMaxConsumptionDay = (dataload: Dataload[]) => {
    let maxIndex = -1
    let maxValue = -1
    dataload.forEach((day, index) => {
      if (day.value > 0 && day.value > maxValue) {
        maxValue = day.value
        maxIndex = index
      }
    })
    if (maxIndex === -1) return null
    return dataload[maxIndex]
  }

  const buttonPrev = () => (
    <IconButton
      aria-label={t('consumption.accessibility.button_previous_value')}
      onClick={() => handleFluidChange(-1)}
      className="arrow-prev"
    >
      <Icon icon={LeftArrowIcon} size={24} />
    </IconButton>
  )

  const buttonNext = () => (
    <IconButton
      aria-label={t('consumption.accessibility.button_next_value')}
      onClick={() => handleFluidChange(1)}
      className="arrow-next"
    >
      <Icon icon={RightArrowIcon} size={24} />
    </IconButton>
  )

  return (
    <div className="max-consumption-container" ref={containerRef}>
      <StyledIcon icon={GraphIcon} size={38} />
      <div className="text-16-normal title">{t('analysis.max_day')}</div>
      <div className="fluid-navigation">
        {fluidTypes.length > 1 && buttonPrev()}
        <div className={`text-20-bold fluid ${fluidColor}`}>
          {t(`FLUID.${currentFluidType}.LABEL`)}
        </div>
        {fluidTypes.length > 1 && buttonNext()}
      </div>

      <div className="data-container">
        {isLoading && (
          <div className="loaderContainer">
            <Loader fluidType={fluidTypes[index]} />
          </div>
        )}
        {!isLoading && (
          <>
            {!maxDayData && (
              <p className={`text-20-bold no_data`}>{t('analysis.no_data')}</p>
            )}
            {maxDayData && (
              <>
                <div className="text-24-bold maxDay-date">
                  {maxDayData.date.setLocale('fr').toFormat('cccc dd LLLL')}
                </div>
                <div>
                  <DataloadSection
                    dataload={maxDayData}
                    fluidType={fluidTypes[index]}
                    dataloadSectionType={DataloadSectionType.NO_COMPARE}
                    toggleEstimationModal={() => null}
                  />
                </div>
                <BarChart
                  chartData={chartData}
                  fluidType={fluidTypes[index]}
                  timeStep={TimeStep.DAY}
                  height={height}
                  width={width}
                  isSwitching={false}
                  clickable={false}
                />
              </>
            )}
          </>
        )}
      </div>
    </div>
  )
}

export default MaxConsumptionCard
