import { Tab, Tabs } from '@material-ui/core'
import classNames from 'classnames'
import Content from 'components/Content/Content'
import EcogestureList from 'components/Ecogesture/EcogestureList'
import CozyBar from 'components/Header/CozyBar'
import Header from 'components/Header/Header'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { EcogestureTab } from 'enum/ecogesture.enum'
import { Ecogesture } from 'models'
import React, { Dispatch, useCallback, useEffect, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import { useLocation, useNavigate } from 'react-router-dom'
import EcogestureService from 'services/ecogesture.service'
import { AppActionsTypes, AppStore } from 'store'
import { updateProfile } from 'store/profile/profile.actions'
import EcogestureEmptyList from './EcogestureEmptyList'
import EcogestureInitModal from './EcogestureInitModal'
import EcogestureReinitModal from './EcogestureReinitModal'
import './ecogestureView.scss'

interface TabPanelProps {
  children: React.ReactNode
  tab: EcogestureTab
  value: number
}
const TabPanel = ({ children, tab, value }: TabPanelProps) => {
  return (
    <div
      role="tabpanel"
      hidden={value !== tab}
      id={`simple-tabpanel-${tab}`}
      aria-labelledby={`simple-tab-${tab}`}
    >
      {children}
    </div>
  )
}

const EcogestureView = () => {
  const [headerHeight, setHeaderHeight] = useState<number>(0)
  const defineHeaderHeight = (height: number) => {
    setHeaderHeight(height)
  }
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const tab = new URLSearchParams(useLocation().search).get('tab')

  const { profile, profileEcogesture, profileType } = useSelector(
    (state: AppStore) => state.ecolyo
  )

  const [tabValue, setTabValue] = useState<EcogestureTab>(
    tab ? parseInt(tab) : EcogestureTab.OBJECTIVE
  )
  const navigate = useNavigate()
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [allEcogestureList, setAllEcogestureList] = useState<Ecogesture[]>([])
  const [doingEcogestureList, setDoingEcogestureList] = useState<Ecogesture[]>(
    []
  )
  const [objectiveEcogestureList, setObjectiveEcogestureList] = useState<
    Ecogesture[]
  >([])
  const [totalViewed, setTotalViewed] = useState<number>(0)
  const [totalAvailable, setTotalAvailable] = useState<number>(0)
  const [openEcogestureInitModal, setOpenEcogestureInitModal] =
    useState<boolean>(!profile.haveSeenEcogestureModal)
  const [openEcogestureReinitModal, setOpenEcogestureReinitModal] =
    useState<boolean>(false)

  const handleReinitClick = useCallback(() => {
    setOpenEcogestureReinitModal(true)
  }, [])

  const handleLaunchForm = useCallback(async () => {
    dispatch(updateProfile({ haveSeenEcogestureModal: true }))
    setOpenEcogestureInitModal(false)
    navigate('/ecogesture-form?modal=false')
  }, [dispatch, navigate])

  const handleCloseEcogestureInitModal = useCallback(async () => {
    dispatch(updateProfile({ haveSeenEcogestureModal: true }))
    setOpenEcogestureInitModal(false)
  }, [dispatch])

  const handleLaunchReinit = useCallback(async () => {
    setOpenEcogestureReinitModal(false)
    setIsLoading(true)
    const ecogestureService = new EcogestureService(client)
    const reset = await ecogestureService.reinitAllEcogestures()
    if (reset) {
      setOpenEcogestureReinitModal(false)
      setIsLoading(false)
      navigate('/ecogesture-form?modal=true')
    }
  }, [client, navigate])

  const handleCloseEcogestureReinitModal = useCallback(() => {
    setOpenEcogestureReinitModal(false)
  }, [])

  const handleChange = useCallback(
    (event: React.ChangeEvent<object>, newValue: number) => {
      event.preventDefault()
      const params = new URLSearchParams()
      params.append('tab', newValue.toString())
      navigate({ search: params.toString() })
      setTabValue(newValue)
    },
    [navigate]
  )

  const tabProps = useCallback((index: number) => {
    return {
      id: `simple-tab-${index}`,
      'aria-controls': `simple-tabpanel-${index}`,
    }
  }, [])

  const getTabLabel = useCallback(
    (tab: EcogestureTab) => {
      const tabCounts = {
        [EcogestureTab.OBJECTIVE]: objectiveEcogestureList.length,
        [EcogestureTab.DOING]: doingEcogestureList.length,
        [EcogestureTab.ALL]: allEcogestureList.length,
      }

      return (
        <>
          {t(`ecogesture.title_tab_${tab}`)}
          <br />
          {`(${tabCounts[tab]})`}
        </>
      )
    },
    [
      allEcogestureList.length,
      doingEcogestureList.length,
      objectiveEcogestureList.length,
      t,
    ]
  )

  useEffect(() => {
    let subscribed = true
    async function loadEcogestures() {
      const ecogestureService = new EcogestureService(client)

      const { ecogestureList, ecogestureHash } =
        await ecogestureService.initEcogesture(profile.ecogestureHash)

      if (ecogestureHash !== profile.ecogestureHash) {
        dispatch(updateProfile({ ecogestureHash }))
      }

      const availableList = await ecogestureService.getEcogestureListByProfile(
        profileEcogesture
      )
      const filteredList = availableList.filter(
        ecogesture => ecogesture.viewedInSelection === false
      )
      if (subscribed && ecogestureList) {
        const doing = ecogestureList.filter(
          ecogesture => ecogesture.doing === true
        )
        const objective = ecogestureList.filter(
          ecogesture => ecogesture.objective === true
        )
        setAllEcogestureList(ecogestureList)
        setObjectiveEcogestureList(objective)
        setDoingEcogestureList(doing)
        setTotalAvailable(availableList.length)
        setTotalViewed(availableList.length - filteredList.length)
      }
      setIsLoading(false)
    }
    loadEcogestures()
    return () => {
      subscribed = false
    }
  }, [client, profileEcogesture, profileType, dispatch, profile.ecogestureHash])

  return (
    <>
      <CozyBar titleKey={'common.title_ecogestures'} />
      {isLoading && (
        <Content height={headerHeight}>
          <div className="ecogesture-spinner" aria-busy="true">
            <Loader text={t('ecogestures.loading')} />
          </div>
        </Content>
      )}
      {!isLoading && (
        <>
          <Header
            setHeaderHeight={defineHeaderHeight}
            desktopTitleKey={'common.title_ecogestures'}
          >
            <Tabs
              value={tabValue}
              className="ecogestures-tabs"
              aria-label="ecogestures-tabs"
              onChange={handleChange}
              TabIndicatorProps={{ className: 'indicator-tab' }}
              centered={true}
            >
              <Tab
                label={getTabLabel(EcogestureTab.OBJECTIVE)}
                className={classNames('single-tab', {
                  ['active']: tabValue === EcogestureTab.OBJECTIVE,
                })}
                {...tabProps(EcogestureTab.OBJECTIVE)}
              />
              <Tab
                label={getTabLabel(EcogestureTab.DOING)}
                className={classNames('single-tab', {
                  ['active']: tabValue === EcogestureTab.DOING,
                })}
                {...tabProps(EcogestureTab.DOING)}
              />
              <Tab
                label={getTabLabel(EcogestureTab.ALL)}
                className={classNames('single-tab', {
                  ['active']: tabValue === EcogestureTab.ALL,
                })}
                {...tabProps(EcogestureTab.ALL)}
              />
            </Tabs>
          </Header>

          <Content height={headerHeight}>
            <TabPanel value={tabValue} tab={EcogestureTab.OBJECTIVE}>
              {profile.isProfileEcogestureCompleted &&
                (totalAvailable === totalViewed &&
                objectiveEcogestureList.length === 0 ? (
                  <EcogestureEmptyList
                    setTab={setTabValue}
                    isObjective={true}
                    isSelectionDone={true}
                    handleReinitClick={handleReinitClick}
                  />
                ) : (
                  <EcogestureList
                    list={objectiveEcogestureList}
                    displaySelection={totalAvailable !== totalViewed}
                    selectionTotal={totalAvailable}
                    selectionViewed={totalViewed}
                    handleReinitClick={handleReinitClick}
                  />
                ))}
              {!profile.isProfileEcogestureCompleted && (
                <EcogestureEmptyList
                  setTab={setTabValue}
                  isObjective={true}
                  isSelectionDone={false}
                  handleReinitClick={handleReinitClick}
                />
              )}
            </TabPanel>

            <TabPanel value={tabValue} tab={EcogestureTab.DOING}>
              {profile.isProfileEcogestureCompleted &&
                (totalAvailable === totalViewed &&
                doingEcogestureList.length === 0 ? (
                  <EcogestureEmptyList
                    setTab={setTabValue}
                    isObjective={false}
                    isSelectionDone={true}
                    handleReinitClick={handleReinitClick}
                  />
                ) : (
                  <EcogestureList
                    list={doingEcogestureList}
                    displaySelection={totalAvailable !== totalViewed}
                    selectionTotal={totalAvailable}
                    selectionViewed={totalViewed}
                    handleReinitClick={handleReinitClick}
                  />
                ))}
              {!profile.isProfileEcogestureCompleted && (
                <EcogestureEmptyList
                  setTab={setTabValue}
                  isObjective={false}
                  isSelectionDone={false}
                  handleReinitClick={handleReinitClick}
                />
              )}
            </TabPanel>

            <TabPanel value={tabValue} tab={EcogestureTab.ALL}>
              {Boolean(allEcogestureList.length) && (
                <EcogestureList
                  list={allEcogestureList}
                  displaySelection={false}
                  selectionTotal={totalAvailable}
                  selectionViewed={totalViewed}
                />
              )}
            </TabPanel>
          </Content>
        </>
      )}
      {openEcogestureInitModal && (
        <EcogestureInitModal
          open={openEcogestureInitModal}
          handleCloseClick={handleCloseEcogestureInitModal}
          handleLaunchForm={handleLaunchForm}
        />
      )}
      {openEcogestureReinitModal && (
        <EcogestureReinitModal
          open={openEcogestureReinitModal}
          handleCloseClick={handleCloseEcogestureReinitModal}
          handleLaunchReinit={handleLaunchReinit}
        />
      )}
    </>
  )
}

export default EcogestureView
