import classNames from 'classnames'
import ExpiredConsentModal from 'components/Connection/ExpiredConsentModal'
import Content from 'components/Content/Content'
import CustomPopupModal from 'components/CustomPopup/CustomPopupModal'
import DateNavigator from 'components/DateNavigator/DateNavigator'
import FluidChart from 'components/FluidChart/FluidChart'
import CozyBar from 'components/Header/CozyBar'
import Header from 'components/Header/Header'
import ConsumptionDetails from 'components/Home/ConsumptionDetails'
import FluidButtons from 'components/Home/FluidButtons'
import KonnectorViewerCard from 'components/Konnector/KonnectorViewerCard'
import KonnectorViewerList from 'components/Konnector/KonnectorViewerList'
import Loader from 'components/Loader/Loader'
import PartnerIssueModal from 'components/PartnerIssue/PartnerIssueModal'
import { useClient } from 'cozy-client'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import React, { Dispatch, useCallback, useEffect, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import { useNavigate } from 'react-router-dom'
import ProfileService from 'services/profile.service'
import { AppActionsTypes, AppStore } from 'store'
import { setCurrentTimeStep, setShowOfflineData } from 'store/chart/chart.slice'
import { showReleaseNotes } from 'store/global/global.actions'
import { openPartnersModal, setCustomPopup } from 'store/modal/modal.slice'
import {
  getKonnectorUpdateError,
  getTodayDate,
  isKonnectorActive,
} from 'utils/utils'
import ReleaseNotesModal from './ReleaseNotesModal'
import './consumptionView.scss'

const ConsumptionView = ({ fluidType }: { fluidType: FluidType }) => {
  const navigate = useNavigate()
  const client = useClient()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const isMulti = fluidType === FluidType.MULTIFLUID
  const {
    chart: { currentTimeStep, loading, showOfflineData },
    global: { fluidStatus, releaseNotes },
    modal: { partnersIssueModal, customPopupModal },
  } = useSelector((state: AppStore) => state.ecolyo)

  const [openReleaseNoteModal, setOpenReleaseNoteModal] = useState<boolean>(
    releaseNotes.show
  )

  const [headerHeight, setHeaderHeight] = useState<number>(0)
  const [active, setActive] = useState<boolean>(false)
  const [openExpiredConsentModal, setOpenExpiredConsentModal] =
    useState<boolean>(true)
  const [consentExpiredFluids, setConsentExpiredFluids] = useState<FluidType[]>(
    []
  )

  const updateKey =
    fluidType !== FluidType.MULTIFLUID && fluidStatus[fluidType].lastDataDate
      ? `${fluidStatus[fluidType].lastDataDate!.toLocaleString()} + ${
          fluidStatus[fluidType].status + fluidType
        }`
      : ''
  const lastDataDateKey =
    fluidType !== FluidType.MULTIFLUID && fluidStatus[fluidType].lastDataDate
      ? `${fluidStatus[fluidType].lastDataDate!.toLocaleString() + fluidType}`
      : ''

  const defineHeaderHeight = useCallback((height: number) => {
    setHeaderHeight(height)
  }, [])

  const handleCloseReleaseNoteModal = useCallback(() => {
    setOpenReleaseNoteModal(false)
    dispatch(
      showReleaseNotes(false, releaseNotes.notes, releaseNotes.redirectLink)
    )
    if (releaseNotes.redirectLink) {
      navigate(releaseNotes.redirectLink)
    }
  }, [dispatch, navigate, releaseNotes.notes, releaseNotes.redirectLink])

  const getPartnerKey = (fluidType: FluidType): 'enedis' | 'egl' | 'grdf' => {
    switch (fluidType) {
      case FluidType.ELECTRICITY:
        return 'enedis'
      case FluidType.WATER:
        return 'egl'
      case FluidType.GAS:
        return 'grdf'
      default:
        throw new Error('unknown fluidtype')
    }
  }

  const handleClosePartnerIssueModal = useCallback(
    async (fluidType: FluidType) => {
      const profileService = new ProfileService(client)
      const profileValues = await profileService.getProfile()
      if (profileValues) {
        const updatedProfile = await profileService.updateProfile({
          partnersIssueSeenDate: {
            ...profileValues.partnersIssueSeenDate,
            [getPartnerKey(fluidType)]: getTodayDate(),
          },
        })
        if (updatedProfile) {
          dispatch(
            openPartnersModal({
              ...partnersIssueModal,
              [getPartnerKey(fluidType)]: false,
            })
          )
        }
      }
    },
    [client, dispatch, partnersIssueModal]
  )

  const handleCloseCustomPopupModal = async () => {
    const profileService = new ProfileService(client)
    const updatedProfile = await profileService.updateProfile({
      customPopupDate: getTodayDate(),
    })
    if (updatedProfile) {
      dispatch(
        setCustomPopup({
          ...customPopupModal,
          popupEnabled: false,
        })
      )
    }
  }

  /** Handle time change */
  useEffect(() => {
    if (
      fluidType !== FluidType.ELECTRICITY &&
      currentTimeStep == TimeStep.HALF_AN_HOUR
    ) {
      dispatch(setCurrentTimeStep(TimeStep.WEEK))
    }
  }, [dispatch, fluidType, currentTimeStep])

  /**
   * If fluid is not connected, display Connect components
   * If fluid is connected, dispatch FluidChart
   */
  useEffect(() => {
    const isFluidConnected = isKonnectorActive(fluidStatus, fluidType)
    dispatch(setShowOfflineData(isFluidConnected))
  }, [dispatch, fluidStatus, fluidType])

  useEffect(() => {
    let subscribed = true
    const expiredConsents: FluidType[] = []
    // Check if some fluids have expired consent error
    for (const fluid of fluidStatus) {
      const error = fluid.connection.triggerState?.last_error
      if (error && getKonnectorUpdateError(error) === 'error_update_oauth') {
        expiredConsents.push(fluid.fluidType)
      }
    }

    if (subscribed) setConsentExpiredFluids(expiredConsents)
    return () => {
      subscribed = false
    }
  }, [fluidStatus])

  return (
    <>
      <CozyBar titleKey={'common.title_consumption'} />
      <Header
        setHeaderHeight={defineHeaderHeight}
        desktopTitleKey={'common.title_consumption'}
      >
        <DateNavigator />
      </Header>
      <Content height={headerHeight}>
        <FluidButtons activeFluid={fluidType} key={updateKey} />

        {openReleaseNoteModal && (
          <ReleaseNotesModal
            open={openReleaseNoteModal}
            handleCloseClick={handleCloseReleaseNoteModal}
          />
        )}
        {showOfflineData && (
          <>
            {loading && (
              <div className={'consumptionview-loading'} aria-busy="true">
                <Loader fluidType={fluidType} />
              </div>
            )}
            <div
              className={classNames('consumptionview-content', {
                ['--hidden']: loading,
              })}
            >
              <FluidChart
                fluidType={fluidType}
                setActive={setActive}
                key={lastDataDateKey}
              />
              <ConsumptionDetails fluidType={fluidType} />
            </div>
            {!isMulti && (
              <div className="konnector-section">
                <KonnectorViewerCard
                  fluidStatus={fluidStatus[fluidType]}
                  fluidType={fluidType}
                  isParam={true}
                  isDisconnected={false}
                  showOfflineData={true}
                  setActive={setActive}
                  active={active}
                  key={fluidType}
                />
              </div>
            )}
          </>
        )}
        {!showOfflineData && (
          <div className="konnector-section">
            {isMulti ? (
              <KonnectorViewerList />
            ) : (
              <KonnectorViewerCard
                fluidStatus={fluidStatus[fluidType]}
                fluidType={fluidType}
                isParam={false}
                isDisconnected={true}
                showOfflineData={false}
                setActive={setActive}
                active={active}
              />
            )}
          </div>
        )}
      </Content>
      {/* Partner issue modals for individual fluids */}
      {fluidStatus
        .filter(fluid => fluid.maintenance)
        .filter(fluid => fluid.fluidType === fluidType)
        .map(issuedFluid => (
          <PartnerIssueModal
            key={issuedFluid.fluidType}
            issuedFluid={issuedFluid}
            open={partnersIssueModal[getPartnerKey(issuedFluid.fluidType)]}
            handleCloseClick={handleClosePartnerIssueModal}
          />
        ))}
      <CustomPopupModal
        customPopup={customPopupModal}
        handleCloseClick={handleCloseCustomPopupModal}
      />
      {Boolean(consentExpiredFluids.length) &&
        consentExpiredFluids.map(fluid => {
          return (
            <ExpiredConsentModal
              key={fluid}
              open={openExpiredConsentModal}
              handleCloseClick={() => setOpenExpiredConsentModal(false)}
              fluidType={fluid}
              toggleModal={() => setOpenExpiredConsentModal(prev => !prev)}
            />
          )
        })}
    </>
  )
}

export default ConsumptionView
