import Button from '@material-ui/core/Button'
import CloseIcon from 'assets/icons/ico/close.svg'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import Loader from 'components/Loader/Loader'
import QuizExplanationModal from 'components/Quiz/QuizExplanationModal'
import { Client, useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { UsageEventType } from 'enum/usageEvent.enum'
import { UserChallengeUpdateFlag } from 'enum/userChallenge.enum'
import { Answer, QuestionEntity, UserChallenge, UserQuiz } from 'models'
import React, { Dispatch, useState } from 'react'
import { useDispatch } from 'react-redux'
import ChallengeService from 'services/challenge.service'
import QuizService from 'services/quiz.service'
import UsageEventService from 'services/usageEvent.service'
import { AppActionsTypes } from 'store'
import { updateUserChallengeList } from 'store/challenge/challenge.slice'
import './quizQuestion.scss'

interface QuizCustomQuestionContent {
  userChallenge: UserChallenge
  goBack: () => void
  question: QuestionEntity | undefined
  isLoading: boolean
}

const QuizCustomQuestionContent = ({
  userChallenge,
  goBack,
  question,
  isLoading,
}: QuizCustomQuestionContent) => {
  const { t } = useI18n()
  const [userChoice, setUserChoice] = useState<string>('')
  const [openModal, setOpenModal] = useState<boolean>(false)
  const [answerIndex, setAnswerIndex] = useState<number>(0)
  const client: Client = useClient()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()

  const quizService: QuizService = new QuizService(client)
  const challengeService: ChallengeService = new ChallengeService(client)

  const validateQuestion = async () => {
    if (question) {
      const resultIndex: number = question.answers.findIndex(
        answer => answer.answerLabel === userChoice
      )
      setAnswerIndex(resultIndex)
      setOpenModal(true)

      const result: Answer[] = question.answers.filter(
        answer => answer.answerLabel === userChoice
      )
      const quizUpdated: UserQuiz = await quizService.updateUserQuiz(
        userChallenge.quiz,
        result[0].isTrue
      )
      const userChallengeUpdated: UserChallenge =
        await challengeService.updateUserChallenge(
          userChallenge,
          UserChallengeUpdateFlag.QUIZ_UPDATE,
          quizUpdated
        )
      dispatch(updateUserChallengeList(userChallengeUpdated))
    }
  }

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setUserChoice(e.target.value)
  }

  const finishQuiz = async () => {
    setOpenModal(false)
    const userChallengeUpdated: UserChallenge =
      await challengeService.updateUserChallenge(
        userChallenge,
        UserChallengeUpdateFlag.QUIZ_DONE
      )
    await UsageEventService.addEvent(client, {
      type: UsageEventType.QUIZ_END_EVENT,
      startDate: userChallenge.quiz.startDate,
      target: userChallenge.quiz.id,
      result: userChallenge.quiz.result.toString(),
      context: userChallenge.id,
    })
    dispatch(updateUserChallengeList(userChallengeUpdated))
  }

  return (
    <div className="quiz-container">
      <div className="question-container">
        <StyledIconButton
          className="btn-back"
          icon={CloseIcon}
          onClick={goBack}
        />
        <p className="index-question">5/5</p>
        <p className="question-title text-18-bold">
          {t('quiz.consumption_question')}
        </p>

        {isLoading ? (
          <div className={'question-loading'} aria-busy="true">
            <Loader />
          </div>
        ) : (
          question && (
            <>
              <p className="question text-18-bold">{question.questionLabel}</p>
              {question.answers.map((answer, index) => {
                return (
                  <div className="answer" key={index}>
                    <input
                      type="radio"
                      id={`answer${index}`}
                      value={answer.answerLabel}
                      onChange={handleChange}
                      checked={userChoice === answer.answerLabel}
                    />
                    <label htmlFor={`answer${index}`} className="text-16-bold">
                      {answer.answerLabel}
                    </label>
                  </div>
                )
              })}
            </>
          )
        )}
      </div>
      <Button
        aria-label={t('quiz.accessibility.button_validate')}
        onClick={validateQuestion}
        disabled={!userChoice}
        classes={{
          root: 'btn-secondary-negative',
          label: 'text-16-normal',
        }}
      >
        {t('quiz.button_validate')}
      </Button>

      <QuizExplanationModal
        open={openModal}
        answerIndex={answerIndex}
        question={question}
        goNext={finishQuiz}
        handleCloseClick={() => setOpenModal(false)}
      />
    </div>
  )
}

export default QuizCustomQuestionContent
