import * as Sentry from '@sentry/react'
import {
  Client,
  MongoSelector,
  Q,
  QueryDefinition,
  QueryResult,
} from 'cozy-client'
import logger from 'cozy-logger'
import { USAGEEVENT_DOCTYPE } from 'doctypes'
import { UsageEventType } from 'enum/usageEvent.enum'
import { DateTime } from 'luxon'
import {
  AddEventParams,
  UsageEvent,
  UsageEventCreationEntity,
  UsageEventEntity,
} from 'models'
import logApp from 'utils/logger'

const logStack = logger.namespace('usageEventService')

export default class UsageEventService {
  /**
   * addEvent
   * @param {Client} client
   * @param {AddEventParams} params
   * @returns {Promise<UsageEvent>} usageEvent added
   */
  static async addEvent(
    client: Client,
    params: AddEventParams
  ): Promise<UsageEvent> {
    const usageEvent: UsageEventCreationEntity = {
      ...params,
      eventDate: DateTime.local()
        .setZone('utc', {
          keepLocalTime: true,
        })
        .toString(),
      startDate: params.startDate ? params.startDate.toString() : undefined,
      aggregated: false,
    }
    const { data }: QueryResult<UsageEventEntity> = await client.create(
      USAGEEVENT_DOCTYPE,
      usageEvent
    )
    return this.parseUsageEventEntityToUsageEvent(data)
  }

  /**
   * addEvent
   * @param {Client} client
   * @param {AddEventParams} params
   * @returns {Promise<UsageEvent>} usageEvent added
   */
  static async addEventIfDoesntExist(
    client: Client,
    params: AddEventParams,
    filterParams: MongoSelector
  ): Promise<UsageEvent | null> {
    const events = await this.getEvents(client, filterParams)
    if (events.length <= 0) {
      return this.addEvent(client, params)
    }
    return null
  }

  /**
   *
   * @param {Client} client
   * @param {string} konnectorSlug
   * @returns
   */
  static async udpateConnectionAttemptEvent(
    client: Client,
    konnectorSlug: string
  ): Promise<UsageEventEntity | undefined> {
    try {
      // Get last Connection attempt Event
      const query: QueryDefinition = Q(USAGEEVENT_DOCTYPE)
        .where({
          target: konnectorSlug,
        })
        .partialIndex({
          type: UsageEventType.KONNECTOR_ATTEMPT_EVENT,
          result: 'error',
        })
        .indexFields(['eventDate', 'type'])
        .sortBy([{ eventDate: 'desc' }])
        .limitBy(1)
      const {
        data: [usageEventEntity],
      }: QueryResult<UsageEventEntity[]> = await client.query(query)
      if (usageEventEntity) {
        const updatedEvent: UsageEventEntity = {
          ...usageEventEntity,
          result: 'success',
        }
        const { data: savedEvent } = await client.save(updatedEvent)
        return savedEvent
      }
    } catch (error) {
      const errorMessage = `UsageEvent service error on udpateConnectionAttemptEvent: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
    }
  }

  /**
   * updateUsageEventsAggregated
   * @param {Client} client
   * @param {string[]} ids
   * @returns {Promise<boolean>} return true if all events are updated
   */
  static async updateUsageEventsAggregated(
    client: Client,
    events: UsageEvent[]
  ): Promise<boolean> {
    for (const event of events) {
      try {
        await client.save({
          ...event,
          aggregated: true,
        })
      } catch (error) {
        const errorMessage = `UsageEvent service error on updateUsageEventsAggregated: ${JSON.stringify(
          error
        )}`
        logStack('error', errorMessage)
        logApp.error(errorMessage)
        Sentry.captureException(errorMessage)
      }
    }
    return true
  }

  /**
   * getEvents
   * @param {Client} client
   * @param {MongoSelector} filterParams
   * @returns {Promise<UsageEvent[]>} usageEvent added
   */
  static async getEvents(
    client: Client,
    filterParams: MongoSelector,
    desc = false
  ): Promise<UsageEvent[]> {
    const query: QueryDefinition = Q(USAGEEVENT_DOCTYPE)
      .where(filterParams)
      .sortBy([{ eventDate: desc ? 'desc' : 'asc' }])
    const { data: usageEventEntities }: QueryResult<UsageEventEntity[]> =
      await client.query(query)
    const usageEvents: UsageEvent[] = usageEventEntities.map(
      (usageEventEntity: UsageEventEntity) => {
        return this.parseUsageEventEntityToUsageEvent(usageEventEntity)
      }
    )
    return usageEvents
  }

  /**
   * Retrieve UsageEvent from the UsageEventEntity
   * @param {UsageEventEntity} usageEventEntity
   * @returns {UsageEvent}
   */
  static parseUsageEventEntityToUsageEvent(
    usageEventEntity: UsageEventEntity
  ): UsageEvent {
    const usageEvent: UsageEvent = {
      ...usageEventEntity,
      eventDate: DateTime.fromISO(usageEventEntity.eventDate, {
        zone: 'utc',
      }),
      startDate: usageEventEntity.startDate
        ? DateTime.fromISO(usageEventEntity.startDate, {
            zone: 'utc',
          })
        : undefined,
    }
    return usageEvent
  }
}
