import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import { FluidType } from 'enum/fluid.enum'
import get from 'lodash/get'
import { DateTime } from 'luxon'
import mjml2html from 'mjml'
import ConsumptionService from 'services/consumption.service'
import EnvironmentService from 'services/environment.service'
import MailService from 'services/mail.service'
import ProfileService from 'services/profile.service'
import { getMonthName } from 'utils/utils'
import { runService } from './service'
const consumptionLimit = require('notifications/consumptionLimit.hbs')

const logStack = logger.namespace('alert')

interface ConsumptionAlertProps {
  client: Client
}

// Only monitoring WATER fluid for now
const consumptionAlert = async ({ client }: ConsumptionAlertProps) => {
  logStack('info', 'Fetching user profile...')
  const upm = new ProfileService(client)
  const consumptionService = new ConsumptionService(client)
  const userProfil = await upm.getProfile()
  if (
    !userProfil?.sendConsumptionAlert ||
    userProfil.waterDailyConsumptionLimit === 0
  ) {
    logStack(
      'info',
      'End of process - Alert report notification is disabled or lack informations from user profile to run'
    )
    return
  }

  let username = ''

  logStack('info', 'water limit is :' + userProfil.waterDailyConsumptionLimit)

  logStack('info', 'Fetching fluid data...')
  // Retrieve public name from the stack
  const settings = await client
    .getStackClient()
    .fetchJSON('GET', '/settings/instance')
  const publicName = get(settings, 'data.attributes.public_name')
  if (publicName) {
    username = publicName
  }

  // Retrieve link to ecolyo app from the stack
  const apps = await client.getStackClient().fetchJSON('GET', '/apps/ecolyo')
  const appLink = get(apps, 'data.links.related')

  const fetchedData = await consumptionService.getLastDataload(FluidType.WATER)
  let lastDayValue = 0
  let alertDay: DateTime = DateTime.local().setZone('utc', {
    keepLocalTime: true,
  })

  if (fetchedData && fetchedData.length > 0) {
    fetchedData.forEach(element => {
      if (element.value > lastDayValue) {
        lastDayValue = element.value
        alertDay = element.date
      }
    })
  }
  if (lastDayValue <= userProfil.waterDailyConsumptionLimit) {
    logStack(
      'info',
      'End of process - Limit consumption set by the user has not been passed.'
    )
    return
  }

  logStack('info', 'Creation of mail...')
  const mailService = new MailService()

  const environmentService = new EnvironmentService()
  const template = consumptionLimit({
    title: 'Ça déborde !',
    baseUrl: environmentService.getPublicURL(),
    username: username,
    clientUrl: `${appLink}/#/consumption/water`,
    unsubscribeUrl: `${appLink}/#/options`,
    userLimit: userProfil.waterDailyConsumptionLimit,
    limitDate: `${alertDay.day} ${getMonthName(alertDay)}`,
    consumption: lastDayValue,
  })

  const mailData = {
    mode: 'noreply',
    subject: '[Ecolyo] - Ça déborde !',
    parts: [
      {
        type: 'text/html',
        body: mjml2html(template).html,
      },
    ],
  }

  logStack('info', 'Sending mail...')
  mailService.SendMail(client, mailData)
}

runService(consumptionAlert)
