import * as Sentry from '@sentry/react'
import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import {
  ENEDIS_MINUTE_DOCTYPE,
  ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE,
} from 'doctypes'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { union } from 'lodash'
import { DateTime } from 'luxon'
import { Datachart, DataloadEntity, TimePeriod } from 'models'
import { EnedisMonthlyAnalysisData } from 'models/enedisMonthlyAnalysis'
import ConsumptionService from 'services/consumption.service'
import EnedisMonthlyAnalysisDataService from 'services/enedisMonthlyAnalysisData.service'
import { runService } from './service'

const logStack = logger.namespace('report')

interface EnedisMonthlyProps {
  client: Client
}

/**
 * Gets the minimum consumption value in a month
 * @param weekEndValuesArray
 * @param weekValuesArray
 * @returns number
 */
const getMinMonthlyLoad = (
  weekEndValuesArray: number[][],
  weekValuesArray: number[][],
  numberofDaysInMonth: number
): number => {
  const totalArray = union(...weekEndValuesArray, ...weekValuesArray)
  const filteredTotal = totalArray.filter(val => val !== -1 && val !== 0)
  const talonCons = Math.min(...filteredTotal)
  const minCons = talonCons * 48 * numberofDaysInMonth
  logStack('info', `Minimum value is ${minCons} `)
  return minCons
}

/**
 *
 * @param monthlyArray
 * @param dataChart
 * @param isWeekend
 * @returns
 */
const populateArrayWithTotalData = (
  monthlyArray: number[][],
  dataChart: Datachart,
  isWeekend: boolean
) => {
  let halfHourDayData: number[]
  if (isWeekend) {
    // we only keep weekend days
    halfHourDayData = dataChart.actualData
      .filter(day => day.date.weekday === 6 || day.date.weekday === 7)
      .map(day => day.value)
  } else {
    // we only keep weekdays
    halfHourDayData = dataChart.actualData
      .filter(day => day.date.weekday !== 6 && day.date.weekday !== 7)
      .map(day => day.value)
  }
  if (halfHourDayData.length > 0) {
    halfHourDayData.forEach((halfHourValue, index) => {
      if (halfHourValue >= 0) {
        if (!monthlyArray[index]) {
          monthlyArray.push([])
        } else {
          monthlyArray[index] = [...monthlyArray[index], halfHourValue]
        }
      }
    })
  }
}
/**
 * Gets max Power value for a given month
 * @param month
 * @param year
 * @param client
 * @returns
 */
const getMonthMaxPower = async (
  month: number,
  year: number,
  client: Client
) => {
  const emas = new EnedisMonthlyAnalysisDataService(client)
  logStack('info', `Fetching max power for month ${month} of year ${year}`)
  const data = await emas.getMaxPowerByDate(year, month)
  const maxPowerArray: number[] = []
  if (data?.length) {
    for (const day of data) {
      maxPowerArray.push(day.load)
    }
  }
  return Math.max(...maxPowerArray)
}
/**
 * Get the average arrays of half-hour value on a given month
 * @param client
 * @param month
 * @param year
 * @returns {Promise<MonthlyAveragesLoads>}
 */
const getEnedisMonthAnalysisData = async (
  client: Client,
  month: number,
  year: number
): Promise<EnedisMonthlyAnalysisData | undefined> => {
  logStack(
    'info',
    `Getting enedis analysis data for month ${month} of year ${year}`
  )

  const timePeriod = {
    startDate: DateTime.fromObject({ month: month, year: year }).startOf(
      'month'
    ),
    endDate: DateTime.fromObject({ month: month, year: year }).endOf('month'),
  }
  const cs = new ConsumptionService(client)
  const data = await cs.getGraphData(timePeriod, TimeStep.DAY, [
    FluidType.ELECTRICITY,
  ])
  const monthlyAveragesLoads: EnedisMonthlyAnalysisData = {
    weekDaysHalfHourAverageValues: [],
    weekEndDaysHalfHourAverageValues: [],
    minimumLoad: null,
    maxPower: null,
    month: month,
    year: year,
  }
  if (data) {
    // 48 is the number of half-hour entries in a day
    const weekEndValuesArray: number[][] = new Array([])
    const weekValuesArray: number[][] = new Array([])

    for (const day of data.actualData) {
      const timePeriod: TimePeriod = {
        startDate: day.date.startOf('day'),
        endDate: day.date.endOf('day'),
      }
      // for each day, we get its halfHour DataChart
      // so we get 48 entries per day
      const halfHourDayData = await cs.getGraphData(
        timePeriod,
        TimeStep.HALF_AN_HOUR,
        [FluidType.ELECTRICITY]
      )
      if (halfHourDayData) {
        populateArrayWithTotalData(weekEndValuesArray, halfHourDayData, true)
        populateArrayWithTotalData(weekValuesArray, halfHourDayData, false)
      }
    }
    const numberofDaysInMonth = DateTime.fromObject({
      month: month,
      year: year,
    }).daysInMonth
    monthlyAveragesLoads.minimumLoad = getMinMonthlyLoad(
      weekEndValuesArray,
      weekValuesArray,
      numberofDaysInMonth
    )
    const arrAvg = (arr: number[]) =>
      arr.reduce((a, b) => a + b, 0) / arr.length
    // at this point we have an array of sums for each 48 half hour timestep
    // so we calculate the average
    const weekEndAverages: number[] = weekEndValuesArray.map(halfHourArray =>
      arrAvg(halfHourArray)
    )
    // so we calculate the average
    const weekAverages = weekValuesArray.map(halfHourArray =>
      arrAvg(halfHourArray)
    )
    monthlyAveragesLoads.weekDaysHalfHourAverageValues = weekAverages
    monthlyAveragesLoads.weekEndDaysHalfHourAverageValues = weekEndAverages
    monthlyAveragesLoads.maxPower = await getMonthMaxPower(month, year, client)
    return monthlyAveragesLoads
  }
}

/**
 * Synchronize enedis monthly analysis with database depending on if the service has already ran
 * and if the enedis minute tracking has been activated
 * @param {Client} client
 * @returns
 */
const syncEnedisMonthlyAnalysisDataDoctype = async ({
  client,
}: EnedisMonthlyProps) => {
  // SERVICE RUNS ONLY IF ENEDIS MINUTE IS ACTIVATED
  const emas = new EnedisMonthlyAnalysisDataService(client)
  const cs = new ConsumptionService(client)
  const firstMinuteData = (await cs.getFirsDataDateFromDoctype(
    ENEDIS_MINUTE_DOCTYPE
  )) as DataloadEntity[]

  const lastEnedisMonthlyAnalysis = await emas.getLastEnedisMonthlyAnalysis()
  if (firstMinuteData?.[0]) {
    // First creates the analysis of the month - 1
    logStack('info', 'Fetching last Enedis monthly Analysis...')
    const firstMinuteDate = DateTime.fromObject({
      year: firstMinuteData[0].year,
      month: firstMinuteData[0].month,
    }).startOf('month')
    const today = DateTime.local().setZone('utc', {
      keepLocalTime: true,
    })
    const analysisDate: DateTime = today.minus({
      month: 1,
    })

    const data = await getEnedisMonthAnalysisData(
      client,
      analysisDate.month,
      analysisDate.year
    )
    if (data) {
      const created = await emas.createEnedisMonthlyAnalysisData(data)
      if (created) {
        logStack('success', 'Created successfully ! ')
      } else {
        logStack('error', 'Failed to create last Enedis monthly Analysis')
        Sentry.captureException(
          JSON.stringify({
            error: 'Failed to create last Enedis monthly Analysis ',
          })
        )
      }
    }
    logStack('info', 'Getting first enedis half hour data date')

    if (lastEnedisMonthlyAnalysis.length > 0) {
      // If user has more than one entry (already synced), fetch the full history
      const firstEnedisMonthlyAnalysis = (await cs.getFirsDataDateFromDoctype(
        ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE
      )) as EnedisMonthlyAnalysisData[]
      if (
        firstEnedisMonthlyAnalysis[0]?.month === firstMinuteData[0].month &&
        firstEnedisMonthlyAnalysis[0]?.year === firstMinuteData[0].year
      ) {
        logStack('info', 'Every Enedis Analysis already synchronized')
        return
      } else if (firstEnedisMonthlyAnalysis) {
        logStack(
          'info',
          'Doctype is partially completed, fetching all available history'
        )
        const firstEnedisMonthlyAnalysisDate = DateTime.fromObject({
          year: firstEnedisMonthlyAnalysis[0].year,
          month: firstEnedisMonthlyAnalysis[0].month,
        }).startOf('month')
        const diffInmonths = Math.ceil(
          firstEnedisMonthlyAnalysisDate.diff(firstMinuteDate, 'months').months
        )
        for (let i = 1; i < diffInmonths; i++) {
          const analysistoCreate = firstEnedisMonthlyAnalysisDate.minus({
            month: i,
          })
          const data = await getEnedisMonthAnalysisData(
            client,
            analysistoCreate.month,
            analysistoCreate.year
          )
          if (data) {
            await emas.createEnedisMonthlyAnalysisData(data)
          }
        }
      }
    } else {
      // If user only have the last analysis available, fetch one year history
      logStack(
        'info',
        'Doctype is empty, fetching history for one year maximum or until first enedis minute date'
      )
      const maximumDate = analysisDate.minus({ month: 12 }).startOf('month')
      const diffInmonths = Math.ceil(
        firstMinuteDate.diff(maximumDate, 'months').months
      )
      const limitDate = diffInmonths > 0 ? firstMinuteDate : maximumDate
      const diffInMonthsWithLimitDate = Math.ceil(
        analysisDate.diff(limitDate, 'months').months
      )
      for (let i = 1; i < diffInMonthsWithLimitDate; i++) {
        const analysistoCreate = analysisDate.minus({ month: i })
        const data = await getEnedisMonthAnalysisData(
          client,
          analysistoCreate.month,
          analysistoCreate.year
        )
        if (data) {
          await emas.createEnedisMonthlyAnalysisData(data)
        }
      }
    }
  } else {
    logStack(
      'info',
      'Enedis Minute is not activated or there is no data yet in this doctype'
    )
  }
}

runService(syncEnedisMonthlyAnalysisDataDoctype)
